import { Injectable } from '@nestjs/common';
import {
  HealthCheckError,
  HealthIndicator,
  HealthIndicatorResult,
} from '@nestjs/terminus';
import { ICheckRepository } from '../interfaces/check-repository.interface';
import { AdsRepository } from '../../../ad/adapters/secondaries/ads.repository';

@Injectable()
export class RepositoriesHealthIndicatorUseCase extends HealthIndicator {
  private checkRepositories: ICheckRepository[];
  constructor(private readonly adsRepository: AdsRepository) {
    super();
    this.checkRepositories = [adsRepository];
  }
  isHealthy = async (key: string): Promise<HealthIndicatorResult> => {
    try {
      await Promise.all(
        this.checkRepositories.map(
          async (checkRepository: ICheckRepository) => {
            await checkRepository.healthCheck();
          },
        ),
      );
      return this.getStatus(key, true);
    } catch (e: any) {
      throw new HealthCheckError('Repository', {
        repository: e.message,
      });
    }
  };
}
